@@ -29,13 +29,14 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <esd.h>
 #include <sys/types.h>
 
+#include <glib/gi18n.h>
+
+#include <esd.h>
+
 #include <gconf/gconf-client.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-sound.h>
-#include <libgnome/gnome-util.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnomeui/gnome-client.h>
 #include "libsounds/sound-properties.h"
